
///////////////////////////////////////////////////////////////////////////////
//$Date: 2010/06/25 15:58:01 $
//$RCSfile: frame_check.ejava,v $
//$Revision: 1.2 $
///////////////////////////////////////////////////////////////////////////////
//   ____  ____ 
//  /   /\/   / 
// /___/  \  /    Vendor: Xilinx 
// \   \   \/     Version : 1.7
//  \   \         Application : RocketIO Wizard 
//  /   /         Filename : frame_check.v
// /___/   /\     Timestamp : 
// \   \  /  \ 
//  \___\/\___\ 
//
//
// Module FRAME_CHECK
// Generated by Xilinx RocketIO Wizard
//  ***************************************************************************
// (c) Copyright 2006 - 2010 Xilinx, Inc. All rights reserved.
// 
// This file contains confidential and proprietary information
// of Xilinx, Inc. and is protected under U.S. and
// international copyright and other intellectual property
// laws.
// 
// DISCLAIMER
// This disclaimer is not a license and does not grant any
// rights to the materials distributed herewith. Except as
// otherwise provided in a valid license issued to you by
// Xilinx, and to the maximum extent permitted by applicable
// law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
// WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
// AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
// BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
// INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
// (2) Xilinx shall not be liable (whether in contract or tort,
// including negligence, or under any other theory of
// liability) for any loss or damage of any kind or nature
// related to, arising under or in connection with these
// materials, including for any direct, or any indirect,
// special, incidental, or consequential loss or damage
// (including loss of data, profits, goodwill, or any type of
// loss or damage suffered as a result of any action brought
// by a third party) even if such damage or loss was
// reasonably foreseeable or Xilinx had been advised of the
// possibility of the same.
// 
// CRITICAL APPLICATIONS
// Xilinx products are not designed or intended to be fail-
// safe, or for use in any application requiring fail-safe
// performance, such as life-support or safety devices or
// systems, Class III medical devices, nuclear facilities,
// applications related to the deployment of airbags, or any
// other applications that could lead to death, personal
// injury, or severe property or environmental damage
// (individually and collectively, "Critical
// Applications"). Customer assumes the sole risk and
// liability of any use of Xilinx products in Critical
// Applications, subject only to applicable laws and
// regulations governing limitations on product liability.
// 
// THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
// PART OF THIS FILE AT ALL TIMES.

//  ***************************************************************************

`timescale 1ns / 1ps
`define DLY #1


module FRAME_CHECK
(
    // User Interface
    RX_DATA,  

    // System Interface
    USER_CLK,       
    RESET,
    ERROR_COUNT
  
);

// synthesis attribute X_CORE_INFO of FRAME_CHECK is "v4fx_mgtwizard_v1_7, Coregen v12.1";

//***********************************Port Declarations*******************************

   // User Interface
    input   [31:0]     RX_DATA;
    
      // System Interface
    input              USER_CLK;
    input              RESET; 
    output  [7:0]      ERROR_COUNT;


//***************************Internal Register Declarations*************************** 

    reg                align_char_detected_r;
    reg                begin_r;
    reg     [1:0]      counter1_r;
    reg     [31:0]     data_r;
    reg     [7:0]      error_count_r;
    reg                error_detected_r;
    reg     [31:0]     rx_data_r;
    reg                start_toggling_r;
    
 
//*********************************Wire Declarations**********************************
   
    wire    [31:0]     align_char_c;
    wire               align_char_detected_c;
    wire               error_detected_c;
    wire               next_begin_c;
    wire               port_has_align_r;
    wire               start_toggling_c;


//*********************************Main Body of Code**********************************


    //_________________________ Register RXDATA for timing____________________________    

    always @(posedge USER_CLK)
        rx_data_r  <=  `DLY  RX_DATA;

    //___________________________ Assign the alignment char __________________________    

    assign  align_char_c    =  32'h000000bc;  

    //________________________________ State machine ________________________________    
    
    // State registers
    always @(posedge USER_CLK)
        if(RESET)
            {begin_r,start_toggling_r}  <=  `DLY    2'b10;
        else
        begin
            begin_r           <=  `DLY    next_begin_c;
            start_toggling_r  <=  `DLY    start_toggling_c;
        end

    // Next state logic
    assign  next_begin_c          =   begin_r & !align_char_detected_c;

    assign  start_toggling_c      =   (begin_r & align_char_detected_c)
                                      | start_toggling_r;
        
    assign align_char_detected_c  =   (rx_data_r == align_char_c);
    
    //__________________ Synchronize the toggle flop to incoming data ________________    

    always @(posedge USER_CLK)
        align_char_detected_r  <=  `DLY    align_char_detected_c;

    //Counter
    always @(posedge USER_CLK)
        if(RESET || align_char_detected_r)  counter1_r   <=  `DLY    2'd1;
        else                                counter1_r   <=  `DLY    counter1_r + 2'd1;

    assign port_has_align_r = counter1_r[1] & counter1_r[0];
    
    //______________________________ Capture incoming data ___________________________    


    //Capture valid incoming data, right shifted 1 bit for comparison with the next valid
    //incoming data
    always @(posedge USER_CLK)
        if(!port_has_align_r)    
            data_r  <=  `DLY    {rx_data_r[0],rx_data_r[31:1]};
    
    //___________________________ Check incoming data for errors __________________________
         
    
    //An error is detected when valid data from the data register, when right shifted, does not match valid data
    //from the RX port of the MGT
    assign  error_detected_c    =   !port_has_align_r && (rx_data_r != data_r);   
    
    
    //We register the error_detected signal for use with the error counter logic
    always @(posedge USER_CLK)
        if(!start_toggling_r)  
            error_detected_r    <=  `DLY    1'b0;
        else
            error_detected_r    <=  `DLY    error_detected_c;  


    
    //We count the total number of errors we detect. By keeping a count we make it less likely that we will miss
    //errors we did not directly observe. This counter must be reset when it reaches its max value
    always @(posedge USER_CLK)
        if(!start_toggling_r)
            error_count_r       <=  `DLY    8'd0;
        else if(error_detected_r)
            error_count_r       <=  `DLY    error_count_r + 1;
            
    
    
    //Here we connect the lower 8 bits of the count (the MSbit is used only to check when the counter reaches
    //max value) to the module output
    assign  ERROR_COUNT =   error_count_r;
    
    
endmodule          

 
