//////////////////////////////////////////////////////////////////////////////
//$Date: 2010/06/25 15:58:02 $
//$RCSfile: gt11_init_rx.ejava,v $
//$Revision: 1.2 $
////////////////////////////////////////////////////////////////////////////////
//   ____  ____ 
//  /   /\/   / 
// /___/  \  /    Vendor: Xilinx 
// \   \   \/     Version : 1.7
//  \   \         Application : RocketIO Wizard 
//  /   /         Filename : gt11_init_rx.v
// /___/   /\     Timestamp : 
// \   \  /  \ 
//  \___\/\___\ 
//
//
// Module GT11_INIT_RX
// Generated by Xilinx RocketIO Wizard


//  ***************************************************************************
// (c) Copyright 2006 - 2010 Xilinx, Inc. All rights reserved.
// 
// This file contains confidential and proprietary information
// of Xilinx, Inc. and is protected under U.S. and
// international copyright and other intellectual property
// laws.
// 
// DISCLAIMER
// This disclaimer is not a license and does not grant any
// rights to the materials distributed herewith. Except as
// otherwise provided in a valid license issued to you by
// Xilinx, and to the maximum extent permitted by applicable
// law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
// WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
// AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
// BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
// INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
// (2) Xilinx shall not be liable (whether in contract or tort,
// including negligence, or under any other theory of
// liability) for any loss or damage of any kind or nature
// related to, arising under or in connection with these
// materials, including for any direct, or any indirect,
// special, incidental, or consequential loss or damage
// (including loss of data, profits, goodwill, or any type of
// loss or damage suffered as a result of any action brought
// by a third party) even if such damage or loss was
// reasonably foreseeable or Xilinx had been advised of the
// possibility of the same.
// 
// CRITICAL APPLICATIONS
// Xilinx products are not designed or intended to be fail-
// safe, or for use in any application requiring fail-safe
// performance, such as life-support or safety devices or
// systems, Class III medical devices, nuclear facilities,
// applications related to the deployment of airbags, or any
// other applications that could lead to death, personal
// injury, or severe property or environmental damage
// (individually and collectively, "Critical
// Applications"). Customer assumes the sole risk and
// liability of any use of Xilinx products in Critical
// Applications, subject only to applicable laws and
// regulations governing limitations on product liability.
// 
// THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
// PART OF THIS FILE AT ALL TIMES.

//  ***************************************************************************

`timescale 1ns / 1ps

module GT11_INIT_RX #(
    parameter       C_SIMULATION = 0  // set to 1 for simulation
)
(
    input           CLK,
    input           START_INIT,

    input           LOCK,
    input           USRCLK_STABLE,
    input           PCS_ERROR,

    output   reg    PMA_RESET,
    output   reg    SYNC,
    output   reg    PCS_RESET,
    output   reg    READY

);

// synthesis attribute X_CORE_INFO of GT11_INIT_RX is "v4fx_mgtwizard_v1_7, Coregen v12.1";

//********************************Parameter Declarations**********************

//----------------------------------------------------------------------------
// Delays - these numbers are defined by the number of USRCLK needed in each
//          state for each reset.  Refer to the User Guide on the block
//          diagrams on the reset and the required delay.
//----------------------------------------------------------------------------
parameter C_DELAY_PMA_RESET = 5;
parameter C_DELAY_SYNC = 87;
parameter C_DELAY_PCS_RESET = 5;
parameter C_DELAY_LOCK = 16;
parameter C_DELAY_WAIT_PCS = 5;
parameter C_DELAY_WAIT_READY = 64;

parameter C_PCS_ERROR_COUNT = 16;

//----------------------------------------------------------------------------
// GT11 Initialization FSM
//----------------------------------------------------------------------------
parameter C_RESET         = 8'b00000001;
parameter C_PMA_RESET     = 8'b00000010;
parameter C_WAIT_LOCK     = 8'b00000100;
parameter C_SYNC          = 8'b00001000;
parameter C_PCS_RESET     = 8'b00010000;
parameter C_WAIT_PCS      = 8'b00100000;
parameter C_ALMOST_READY  = 8'b01000000;
parameter C_READY         = 8'b10000000;

//*******************************Register Declarations************************

reg           lock_r;
reg           lock_r2;

reg           pcs_error_r1;
reg           pcs_error_r2;

reg   [2:0]   pma_reset_count_r;
reg   [7:0]   sync_count_r;
reg   [2:0]   pcs_reset_count_r;
reg   [3:0]   wait_pcs_count_r;
reg   [4:0]   pcs_error_count_r;
reg   [7:0]   wait_ready_count_r;

reg   [7:0]   init_state_r;
reg   [7:0]   init_next_state_r;

reg [40*7:0]  init_fsm_name;
reg           usrclk_stable_r;

//*******************************Wire Declarations****************************

wire          pcs_reset_c;
wire          pma_reset_c;
wire          pma_reset_done_i;
wire          ready_c;
wire          sync_c;

wire          lock_pulse_i;
wire          lockupdate_ready_i;
wire          stage_1_enable_i;
wire          stage_2_enable_i;
wire          stage_3_enable_i;
wire          shift_register_1_enable_i;
wire          shift_register_2_enable_i;
wire          shift_register_3_enable_i;
wire          shift_register_0_d_i;
wire          shift_register_1_d_i;
wire          shift_register_2_d_i;
wire          shift_register_3_d_i;
wire          shift_register_0_q_i;
wire          shift_register_1_q_i;
wire          shift_register_2_q_i;
wire          shift_register_3_q_i;

wire          sync_done_i;

wire          pcs_reset_done_i;
wire          wait_pcs_done_i;
wire          pcs_error_count_done_i;
wire          wait_ready_done_i;

//*******************************Main Body of Code****************************

//----------------------------------------------------------------------------
// Synchronize LOCK
//----------------------------------------------------------------------------
always @(posedge CLK or posedge START_INIT)
    if (START_INIT)
        lock_r <= 1'b0;
    else
        lock_r <= LOCK;

//----------------------------------------------------------------------------
// Synchronize USRCLK_STABLE
//----------------------------------------------------------------------------
always @(posedge CLK or posedge START_INIT)
    if (START_INIT)
        usrclk_stable_r <= 1'b0;
    else
        usrclk_stable_r <= USRCLK_STABLE;

//----------------------------------------------------------------------------
// Synchronize PCS error and generate a pulse when PCS error is high
//----------------------------------------------------------------------------
always @(posedge CLK or posedge START_INIT)
    if (START_INIT) 
        pcs_error_r1 <= 1'b0;
    else 
        pcs_error_r1 <= PCS_ERROR;

//----------------------------------------------------------------------------
// Ready, PMA and PCS reset signals
//----------------------------------------------------------------------------
always @(posedge CLK)
    PMA_RESET <= pma_reset_c;

always @(posedge CLK)
    SYNC <= sync_c;

always @(posedge CLK)
    PCS_RESET <= pcs_reset_c;

always @(posedge CLK)
    READY <= ready_c;

assign pma_reset_c = (init_state_r == C_PMA_RESET);
assign sync_c = (init_state_r == C_SYNC);
assign pcs_reset_c = (init_state_r == C_PCS_RESET);
assign ready_c = (init_state_r == C_READY);

//----------------------------------------------------------------------------
// Counter for holding PMA reset for an amount of C_DELAY_PMA_RESET
//----------------------------------------------------------------------------
always @(posedge CLK)
begin
    if (!(init_state_r == C_PMA_RESET))
        pma_reset_count_r <= C_DELAY_PMA_RESET;
    else
        pma_reset_count_r <= pma_reset_count_r - 1'b1;
end

assign pma_reset_done_i = (pma_reset_count_r == 1);

generate
    if (C_SIMULATION) begin : for_simulation
        assign lockupdate_ready_i = 1'b1;
    end
    else begin: for_hardware
        //--------------------------------------------------------------------
        // Counter for the LOCKUPDATE cycles for RXLOCK:  This is built with
        // SRL16s.  Currently, we need to wait for 16 LOCKUPDATE cycles to
        // qualify the RXLOCK signal.  Each LOCKUPDATE = 1024 REFCLK cycles.
        // In this module, we assume 1 REFCLK = 1 USRCLK (please adjust the
        // address of the fourth SRL16 stage as neccessary).  The whole four
        // stages give 16,304 cycles of delay (16 * 16 * 16 * 4); note that
        // the last stage has an extra FF at the end.
        //---------------------------------------------------------------------
        // Create a pulse from RXLOCK to initialize SRL16's.  Need to register
        // the LOCK signal from the GT11 twice since the LOCK is based on
        // REFCLK
        always @(posedge CLK)
            lock_r2 <= lock_r; 

        assign lock_pulse_i = lock_r & ~lock_r2;
        
        // SRL16 Stage Zero - First stage of shifting
        assign shift_register_0_d_i = lock_r &
                                      (lock_pulse_i | stage_1_enable_i);

        SRL16E shift_register_0
        (
            .Q    (shift_register_0_q_i),
            .A0   (1'b0),
            .A1   (1'b1),
            .A2   (1'b1),
            .A3   (1'b1),
            .CE   (1'b1),
            .CLK  (CLK),
            .D    (shift_register_0_d_i)
            );

        FDE flop_stage_0
        (
            .Q  (stage_1_enable_i),
            .C  (CLK),
            .CE (1'b1),
            .D  (shift_register_0_q_i)
            );

        // SRL16 Stage One - Second stage of shifting
        assign shift_register_1_d_i = lock_r & (lock_pulse_i |
                                      (stage_1_enable_i & stage_2_enable_i));
        assign shift_register_1_enable_i = !lock_r2 | stage_1_enable_i;

        SRL16E shift_register_1
        (
            .Q    (shift_register_1_q_i),
            .A0   (1'b0),
            .A1   (1'b1),
            .A2   (1'b1),
            .A3   (1'b1),
            .CE   (shift_register_1_enable_i),
            .CLK  (CLK),
            .D    (shift_register_1_d_i)
            );

        FDE flop_stage_1
        (
            .Q  (stage_2_enable_i),
            .C  (CLK),
            .CE (shift_register_1_enable_i),
            .D  (shift_register_1_q_i)
            );

        // SRL16 Stage Two - Third stage of shifting
        assign shift_register_2_d_i = lock_r & (lock_pulse_i |
                                       (stage_1_enable_i & stage_2_enable_i &
                                       stage_3_enable_i));

        assign shift_register_2_enable_i = !lock_r2 |
                                    (stage_1_enable_i & stage_2_enable_i);

        SRL16E shift_register_2
        (
            .Q    (shift_register_2_q_i),
            .A0   (1'b0),
            .A1   (1'b1),
            .A2   (1'b1),
            .A3   (1'b1),
            .CE   (shift_register_2_enable_i),
            .CLK  (CLK),
            .D    (shift_register_2_d_i)
            );

        FDE flop_stage_2
        (
            .Q  (stage_3_enable_i),
            .C  (CLK),
            .CE (shift_register_2_enable_i),
            .D  (shift_register_2_q_i)
            );

        // SRL16 Stage Three - Fourth stage of shifting
        // LOCKUPDATE ready is redundant here in resetting the SRL16 since the
        // flop is already reset by RXLOCK from the MGT
        assign shift_register_3_d_i = lock_r & (lock_pulse_i | 
                                      (stage_1_enable_i & stage_2_enable_i &
                                       stage_3_enable_i & lockupdate_ready_i));

        assign shift_register_3_enable_i = !lock_r2 |
                                    (stage_1_enable_i & stage_2_enable_i &
                                     stage_3_enable_i & !lockupdate_ready_i);

        SRL16E shift_register_3
        (
            .Q    (shift_register_3_q_i),
            .A0   (1'b1),
            .A1   (1'b1),
            .A2   (1'b0),
            .A3   (1'b0),
            .CE   (shift_register_3_enable_i),
            .CLK  (CLK),
            .D    (shift_register_3_d_i)
            );

        FDRE flop_stage_3
        (
            .Q  (lockupdate_ready_i),
            .C  (CLK),
            .CE (shift_register_3_enable_i),
            .D  (shift_register_3_q_i),
            .R  (!lock_r)
            );
    end
endgenerate

//----------------------------------------------------------------------------
// Counter for holding SYNC for an amount of C_DELAY_SYNC
//----------------------------------------------------------------------------
always @(posedge CLK)
begin
    if (!(init_state_r == C_SYNC))
        sync_count_r <= C_DELAY_SYNC;
    else
        sync_count_r <= sync_count_r - 1'b1;
end

assign sync_done_i = (sync_count_r == 1);

//----------------------------------------------------------------------------
// Counter for holding PCS reset for an amount of C_DELAY_PCS_RESET
//----------------------------------------------------------------------------
always @(posedge CLK)
begin
    if (!(init_state_r == C_PCS_RESET))
        pcs_reset_count_r <= C_DELAY_PCS_RESET;
    else
        pcs_reset_count_r <= pcs_reset_count_r - 1'b1;
end

assign pcs_reset_done_i = (pcs_reset_count_r == 1);

//----------------------------------------------------------------------------
// Counter for waiting C_DELAY_WAIT_PCS after de-assertion of PCS reset
//----------------------------------------------------------------------------
always @(posedge CLK)
begin
    if (!(init_state_r == C_WAIT_PCS))
        wait_pcs_count_r <= C_DELAY_WAIT_PCS;
    else
        wait_pcs_count_r <= wait_pcs_count_r - 1'b1;
end

assign wait_pcs_done_i = (wait_pcs_count_r == 1);

//----------------------------------------------------------------------------
// Counter for PCS error
//----------------------------------------------------------------------------
always @(posedge CLK)
begin
    if (init_state_r == C_PMA_RESET)
        pcs_error_count_r <= C_PCS_ERROR_COUNT;
    else if (((init_state_r == C_ALMOST_READY) | (init_state_r == C_READY)) & pcs_error_r1 & lock_r)
        pcs_error_count_r <= pcs_error_count_r - 1'b1;
end

assign pcs_error_count_done_i = (pcs_error_count_r == 1);

//----------------------------------------------------------------------------
// Counter for the READY signal
//----------------------------------------------------------------------------
always @(posedge CLK)
begin
    if (!(init_state_r == C_ALMOST_READY) || pcs_error_r1)
        wait_ready_count_r <= C_DELAY_WAIT_READY;
    else if (~pcs_error_r1)
        wait_ready_count_r <= wait_ready_count_r - 1'b1;
end

assign wait_ready_done_i = (wait_ready_count_r == 1);


//----------------------------------------------------------------------------
// GT11 Initialization FSM - This FSM is used to initialize the GT11 block
//   asserting the PMA and PCS reset in sequence.  It also takes into account
//   of any error that may happen during initialization.  The block uses
//   USRCLK as reference for the delay.  DO NOT use the output of the GT11
//   clocks for this reset module, as the output clocks may change when reset
//   is applied to the GT11.  Use a system clock, and make sure that the
//   wait time for each state equals the specified number of USRCLK cycles.
//
// The following steps are applied:
//   1. C_RESET:  Upon system reset of this block, PMA reset will be asserted
//   2. C_PMA_RESET:  PMA reset is held for 3 USRCLK cycles
//   3. C_WAIT_LOCK:  Wait for LOCK.  After LOCK is asserted, wait for 16
//      LOCKUPDATE cycles and wait for the USRCLK of the GT11s to be stabled
//      before going to the next state to assert the PCS reset.  If LOCK gets
//      de-asserted, we reset the counter and wait for LOCK again.
//   4. C_SYNC:  Assert SYNG for 64 SYNC cycles.  If LOCK gets de-asserted, we
//      go back to Step 3.
//   5. C_PCS_RESET:  Assert PCS reset for 3 USRCLK cycles.  If LOCK gets
//      de-asserted, we go back to Step 3.
//   6. C_WAIT_PCS:  After de-assertion of PCS reset, wait 5 USRCLK cycles.
//      If LOCK gets de-asserted, we go back to Step 3.
//   7. C_ALMOST_READY:  Go to the Almost Ready state.  If LOCK gets
//      de-asserted, we go back to Step 3.  If there is a PCS error
//      (i.e. buffer error) detected while LOCK is high, we go back to Step 4.
//      If we cycle PCS reset for an N number of C_PCS_ERROR_COUNT, we go back
//      to Step 1 to do a PMA reset.
//   8. C_READY:  Go to the Ready state.  We reach this state after waiting
//      64 USRCLK cycles without any PCS errors.  We assert the READY signal
//      to denote that this block finishes initializing the GT11.  If there is
//      a PCS error during this state, we go back to Step 4.  If LOCK is lost,
//      we go back to Step 3.
//----------------------------------------------------------------------------
always @(posedge CLK or posedge START_INIT)
begin
    if (START_INIT)
        init_state_r <= C_RESET;
    else
        init_state_r <= init_next_state_r;
end

always @*
begin
    case (init_state_r)
        C_RESET: begin
            init_next_state_r <= C_PMA_RESET;
            init_fsm_name = "C_RESET";
        end

        C_PMA_RESET: begin
            init_next_state_r <= pma_reset_done_i ? C_WAIT_LOCK : C_PMA_RESET;
            init_fsm_name = "C_PMA_RESET";
        end

        C_WAIT_LOCK: begin
            init_next_state_r <= (lock_r & usrclk_stable_r &
                                  lockupdate_ready_i) ?
                                  C_SYNC : C_WAIT_LOCK;
            init_fsm_name = "C_WAIT_LOCK";
        end

        C_SYNC: begin
            if (lock_r)
                init_next_state_r <= sync_done_i ? C_PCS_RESET : C_SYNC;
            else
                init_next_state_r <= C_WAIT_LOCK;
            init_fsm_name = "C_SYNC";
        end

        C_PCS_RESET: begin
            if (lock_r)
                init_next_state_r <= pcs_reset_done_i ?
                                     C_WAIT_PCS : C_PCS_RESET;
            else
                init_next_state_r <= C_WAIT_LOCK;
            init_fsm_name = "C_PCS_RESET";
        end

        C_WAIT_PCS: begin
            if (lock_r)
                init_next_state_r <= wait_pcs_done_i ?
                                     C_ALMOST_READY : C_WAIT_PCS;
            else
                init_next_state_r <= C_WAIT_LOCK;
            init_fsm_name = "C_WAIT_PCS";
        end

        C_ALMOST_READY: begin
            if (~lock_r)
                init_next_state_r <= C_WAIT_LOCK;
            else if (pcs_error_r1 & ~pcs_error_count_done_i)
                init_next_state_r <= C_SYNC;
            else if (pcs_error_r1 & pcs_error_count_done_i)
                init_next_state_r <= C_PMA_RESET;
            else if (wait_ready_done_i)
                init_next_state_r <= C_READY;
            else
                init_next_state_r <= C_ALMOST_READY;
            init_fsm_name = "C_ALMOST_READY";
        end

        C_READY: begin
              if (lock_r & ~pcs_error_r1)
                init_next_state_r <= C_READY;
              else if (lock_r & pcs_error_r1)
                init_next_state_r <= C_PCS_RESET;
              else
                init_next_state_r <= C_WAIT_LOCK;
            init_fsm_name = "C_READY";
        end

        default: begin
            init_next_state_r <= C_RESET;
            init_fsm_name = "default";
        end

    endcase
end

endmodule
