`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: UMass Amherst
// Engineer: Cory Gorman
// 
// Design Name:    sata_core
// Module Name:    loopback_test 
// Project Name: 
// Target Devices: Virtex-4
// Tool versions: ISE 14.2
// Description: A simple state machine for testing the SATA core. Emits a sequence
// of link layer primitives that exercises some of the TX datapath. This sequnece
// is easy to add to or change.
//
// Dependencies: 
//
// Revision: 
// Revision 1.0
// Additional Comments: 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.  
//
//////////////////////////////////////////////////////////////////////////////////
module loopback_test(
   input          usr_clk,
   input          reset,
   output [31:0]  txdata,
   output [ 3:0]  txcharisk
);



//States
parameter [5:0] s_sync = 5'h0,
					 s_sof = 5'h1,
					 s_eof = 5'h2,
					 s_r_rdy = 5'h3,
					 s_align = 5'h4,
                s_r_ok  = 5'h5;

//Defined primitives
parameter [31:0] SYNC = 32'hB5B5957C,
					  R_RDY = 32'h4A4A957C,
					  R_IP = 32'h5555B57C,
					  R_OK = 32'h3535B57C,
					  X_RDY = 32'h5757B57C,
					  SOF = 32'h3737B57C,
					  EOF = 32'hD5D5B57C,
					  WTRM = 32'h5858B57C,
					  R_ERR = 32'h5656B57C,
					  ALIGN = 32'h7B4A4ABC;
					 
reg [5:0] state, state_next;
reg [31:0] data, data_next;

always @ (*) begin
	state_next = state;
	data_next = data;
	case (state)
		s_sync: begin
			data_next = SYNC;
			state_next = s_sof;
		end
		s_sof: begin
			data_next = SOF;
			state_next = s_r_rdy;
		end
		s_r_rdy: begin
			data_next = R_RDY;
			state_next = s_align;
		end
		s_align: begin
			data_next = ALIGN;
			state_next = s_eof;
		end
		s_eof: begin
			data_next = EOF;
			state_next = s_r_ok;
		end
      s_r_ok: begin
         data_next = R_OK;
         state_next = s_sync;
      end
	endcase
end

always @ (posedge usr_clk or posedge reset) begin
	if (reset) begin
		state <= s_sync;
		data <= 32'h0;
	end
	else begin
		state <= state_next;
		data <= data_next;
	end
end

	assign txdata = data;
	assign txcharisk = 4'b0001;
endmodule
