`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: UMass Amherst
// Engineer: Cory Gorman
// 
// Design Name: 
// Module Name:    oob_sig_gen 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: Wraps the tx_comreset and tx_comwake modules, simplifying their use.
// Sends an OOB signal on send_en assertion. Use send_select=0 to send a COMRESET,
// and send_select=1 to send a COMWAKE.
//
// Dependencies: 
//
// Revision: 
// Revision 2.0
// Additional Comments: 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.  
//
//////////////////////////////////////////////////////////////////////////////////
module oob_sig_gen # (
//Default: 100MHz system clock
	parameter COUNT_106NS 		= 11,
	parameter COUNT_320NS 		= 32
)(
    input clk,
    input reset,
    input send_en,
    input send_select,
    output rdy,
    output txenoob
    );
	 
	 wire cr_send, cw_send;
	 reg cr_lb, cw_lb;
	 wire cr_txenoob, cw_txenoob;
	 wire cr_rdy, cw_rdy;
	 
	 tx_comreset #(
		.COUNT_106NS(COUNT_106NS),
		.COUNT_320NS(COUNT_320NS)
	 ) tx_comreset_i 
	 (
		.clk (clk),
	   .reset (reset),
		.line_busy (1'b0),
		.send_en (cr_send),
		.rdy (cr_rdy),
		.txenoob (cr_txenoob)
	 );
	 
	 tx_comwake #(
		.COUNT_106NS(COUNT_106NS)
	 ) tx_comwake_i 
	 (
		.clk (clk),
	   .reset (reset),
		.line_busy (1'b0),
		.send_en (cw_send),
		.rdy (cw_rdy),
		.txenoob (cw_txenoob)
	 );
	 
	 assign cr_send = (send_en && ~send_select && rdy);
	 assign cw_send = (send_en && send_select && rdy);		
	 
	 //Ready if both are ready
	 assign rdy = cr_rdy && cw_rdy;
	 
	 assign txenoob = (rdy || cr_txenoob || cw_txenoob);


endmodule
