`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: UMass Amherst
// Engineer: Cory Gorman
// 
// Create Date:    13:44:57 05/06/2013 
// Design Name:    sata_core
// Module Name:    sata_test 
// Project Name: 
// Target Devices: Virtex4
// Tool versions: ISE 14.2
// Description: A simple demonstration of the SATA core.
//
// Dependencies: 
//
// Revision: 
// Revision 1.0
// Additional Comments: 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.  
//
//////////////////////////////////////////////////////////////////////////////////
module sata_test(
	 input UPPER_MGTCLK_PAD_N_IN,
    input UPPER_MGTCLK_PAD_P_IN,
    input CLK_100_IN,
    input TX_SYSTEM_RESET_IN,
    input RX_SYSTEM_RESET_IN,
    input [1:0] RX1N_IN,
    input [1:0] RX1P_IN,
    output [1:0] TX1N_OUT,
    output [1:0] TX1P_OUT,
	 
	 input START,
	 output LINKUP,
	 
	 input SERIAL_RX,
	 output SERIAL_TX
    );
	 
wire clk_100;
IBUFG clk_100_bufg (.I(CLK_100_IN), .O(clk_100));

//Start Button Debounce
reg start_d;
reg [31:0] rdc;

always @ (posedge clk_100) 
begin
rdc[31:0] <= {rdc[30:0],START}; //shift register
if (rdc[31:0] == 32'b00000000)
  start_d <= 1'b0;
else if(rdc[31:0] == 32'hFFFFFFFF)
  start_d <= 1'b1;
else start_d <= start_d;
end

//Chipscope
wire [35:0] sata_tx_frame_ila_control, sata_rx_frame_ila_control,
				sata_phy_ila_control, sata_loopback_ila_control, cmd_layer_ila_control,
				debug_log_ila_control, test_ila_control;

//Command wires
wire ready_for_cmd;
wire [31:0] sector_count;
wire sata_core_full;
wire [31:0] sata_data_from_disk;
wire sata_core_empty;
wire sata_write_clk;
wire sata_read_clk;
//Timer
wire [31:0] sata_timer;

//Registers
reg [4:0] state, state_next;
reg new_cmd, new_cmd_next;
reg [1:0] cmd_type, cmd_type_next;
reg sata_data_to_disk_we, sata_data_to_disk_we_next;
reg sata_data_from_disk_re, sata_data_from_disk_re_next;
reg [31:0] sata_data_to_disk, sata_data_to_disk_next;
reg [47:0] cycle_count, cycle_count_next;
reg test_finished, test_finished_next;
reg core_reset, core_reset_next;
reg [31:0] failures, failures_next;
reg [31:0] tries, tries_next;
reg [47:0] total_count, total_count_next;
reg [31:0] sector_addr, sector_addr_next;

parameter [31:0] ATTEMPTS = 100;

parameter [1:0] CMD_WRITE = 2'b10,
					 CMD_READ  = 2'b01;
					 
parameter [4:0] s_idle = 5'b00000,
					 s_write = 5'b00001,
					 s_retry = 5'b00010,
					 s_write_success = 5'b00011;
					 
assign sector_count = 32'h40;

reg ready_for_cmd_r;
always @ (posedge sata_read_clk) ready_for_cmd_r <= ready_for_cmd;

always @ (*)
begin
	state_next = state;
	cmd_type_next = cmd_type;
	sata_data_to_disk_we_next = 1'b0;
	sata_data_from_disk_re_next = 1'b0;
	new_cmd_next = new_cmd;
	sata_data_to_disk_next = sata_data_to_disk;
	cycle_count_next = cycle_count;
	test_finished_next = test_finished;
	core_reset_next = 1'b0;
	failures_next = failures;
	total_count_next = total_count;
	sector_addr_next = sector_addr;
   tries_next = tries;
	
	case (state)
		s_idle: begin
			if (ready_for_cmd_r && start_d) begin
            test_finished_next = 1'b0;
				new_cmd_next = 1'b1;
				cmd_type_next = CMD_WRITE;
				sata_data_to_disk_next = 32'h0;
				cycle_count_next = 0;
				state_next = s_write;
            tries_next = tries + 1;
			end
		end
		
		s_write: begin
			cycle_count_next = cycle_count + 1;
			//Check to see if done with test
         if (ready_for_cmd_r && (sata_data_to_disk >= 100)) begin
					new_cmd_next = 1'b0;
					state_next = s_write_success;
			end
         //Write
			if (~sata_core_full) begin
				sata_data_to_disk_next = sata_data_to_disk + 1;
				sata_data_to_disk_we_next = 1'b1;
			end
			//Timeout; transmission faield
			//if (cycle_count >= 32'h11E1A300) begin //3 seconds without success
         if (cycle_count >= 48'h23C34600) begin //6 sec
				new_cmd_next = 1'b0;
				core_reset_next = 1'b1;
				if (cycle_count >= 48'h23C34633) begin //hold reset
					state_next = s_retry;
					cycle_count_next = 0;
				end
			end
		end
		
		s_retry: begin
			if (ready_for_cmd_r) begin
				new_cmd_next = 1'b1;
				cmd_type_next = CMD_WRITE;
				sata_data_to_disk_next = 32'h0;
				cycle_count_next = 0;
				test_finished_next = 1'b0;
				state_next = s_write;
				failures_next = failures + 1;
            tries_next = tries + 1;
			end
			else begin
				cycle_count_next = cycle_count + 1;
				if (cycle_count >= 48'h23C34600) begin //unsuccessful reset
					//reset again
					core_reset_next = 1'b1;
					if (cycle_count >= 48'h23C34633) begin //hold reset
						cycle_count_next = 0;
					end
				end
			end
		end
		
		s_write_success: begin
			if (tries < ATTEMPTS) begin
				//add this to the total time, so we can produce an average over all successful attempts
				total_count_next = total_count + cycle_count;
				//do another
				new_cmd_next = 1'b1;
				sata_data_to_disk_next = 32'h0;
				cycle_count_next = 0;
				state_next = s_write;
				sector_addr_next = sector_addr + sector_count; //1;
            tries_next = tries + 1;
			end
			else begin
				test_finished_next = 1'b1;
				new_cmd_next = 1'b0;
				state_next = s_idle;
			end
		end			
	endcase
end

always @ (posedge sata_read_clk or posedge TX_SYSTEM_RESET_IN)
begin
	if (TX_SYSTEM_RESET_IN) begin
		state <= s_idle;
		cmd_type <= CMD_READ;
		new_cmd <= 1'b0;
		sata_data_to_disk <= 32'hDEADBEEF;
		cycle_count <= 48'h0;
		sata_data_to_disk_we <= 1'b0;
		sata_data_from_disk_re <= 1'b0;
		test_finished <= 1'b0;
		core_reset <= 1'b0;
		failures <= 32'h0;
		total_count <= 48'h0;
		sector_addr <= 32'h0;
      tries <= 32'h0;
	end
	else begin
		state <= state_next;
		cmd_type <= cmd_type_next;
		new_cmd <= new_cmd_next;
		sata_data_to_disk <= sata_data_to_disk_next;
		cycle_count <= cycle_count_next;
		sata_data_to_disk_we <= sata_data_to_disk_we_next;
		sata_data_from_disk_re <= sata_data_from_disk_re_next;
		test_finished <= test_finished_next;
		core_reset <= core_reset_next;
		failures <= failures_next;
		total_count <= total_count_next;
		sector_addr <= sector_addr_next;
      tries <= tries_next;
	end
end

reg new_cmd_d1, new_cmd_d2, cmd_start;
reg [3:0] cmd_count;
wire core_new_cmd;
always@ (posedge sata_read_clk) begin
	new_cmd_d1 <= new_cmd;
	new_cmd_d2 <= new_cmd_d1;
	cmd_start <= (new_cmd_d2 == 1'b0) & (new_cmd_d1 == 1'b1);
	if (cmd_start || cmd_count != 0)
		cmd_count <= cmd_count + 1;
end
assign core_new_cmd = (cmd_count != 0);
	

sata_core #
(
   .DEBUG(1),
   .USE_REPLAY_BUFFER(1),
   .USE_EVENT_LOGGER (0)
) sata_core_i (
	 .UPPER_MGTCLK_PAD_N_IN (UPPER_MGTCLK_PAD_N_IN),
    .UPPER_MGTCLK_PAD_P_IN (UPPER_MGTCLK_PAD_P_IN),
    .sys_clk (clk_100),
    .reset(TX_SYSTEM_RESET_IN | core_reset),
    .RX1N_IN (RX1N_IN),
    .RX1P_IN (RX1P_IN),
    .TX1N_OUT (TX1N_OUT),
    .TX1P_OUT (TX1P_OUT),
	 
	 .ready_for_cmd (ready_for_cmd),
    .new_cmd (core_new_cmd),
    .cmd_type (cmd_type),
    .sector_count (sector_count),
    .sector_addr (sector_addr),
	 
    .sata_din (sata_data_to_disk),
    .sata_din_we (sata_data_to_disk_we),
    .sata_core_full (sata_core_full),
    .sata_dout (sata_data_from_disk), 
    .sata_dout_re (1'b0),
    .sata_core_empty (sata_core_empty),
    .SATA_USER_DATA_CLK_IN (sata_read_clk),
    .SATA_USER_DATA_CLK_OUT (sata_read_clk),
	 
	 .LINKUP (LINKUP),
	 
	 .sata_phy_ila_control (sata_phy_ila_control),
    .cmd_layer_ila_control (cmd_layer_ila_control),
    .sata_tx_frame_ila_control (sata_tx_frame_ila_control),
	 .sata_rx_frame_ila_control (sata_rx_frame_ila_control),
	 
	 .SERIAL_RX (SERIAL_RX),
	 .SERIAL_TX (SERIAL_TX),
    .sata_timer ()
);

icon5 icon_i
(
	.CONTROL0                         (sata_phy_ila_control),
	.CONTROL1								 (cmd_layer_ila_control),
	.CONTROL2								 (sata_tx_frame_ila_control),
	.CONTROL3								 (sata_rx_frame_ila_control),
	.CONTROL4								 (test_ila_control)
);

test_ila test_ila_i (
    .CONTROL (test_ila_control),
    .CLK (clk_100),
    .TRIG0 (sata_data_to_disk),
    .TRIG1 (failures),
    .TRIG2 ({TX_SYSTEM_RESET_IN, state}),
    .TRIG3 (core_new_cmd),
    .TRIG4 (cmd_type),
    .TRIG5 (ready_for_cmd),
    .TRIG6 (core_reset),
    .TRIG7 (sata_data_to_disk_we),
    .TRIG8 (sata_data_from_disk_re),
    .TRIG9 (test_finished),
	 .TRIG10 (total_count),
	 .TRIG11 (sector_addr)
);

endmodule
