`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: UMass Amherst
// Engineer: Cory Gorman
// 
// Design Name:    sata_core
// Module Name:    tx_comwake
// Project Name: 
// Target Devices: Virtex-4
// Tool versions: 
// Description: Sends COMWAKE OOB signal: six 106ns bursts with 106ns between them.
//
// Dependencies: 
//
// Revision: 
// Revision 1.0
// Additional Comments: 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.  
//
//////////////////////////////////////////////////////////////////////////////////
module tx_comwake # (
//See tx_comreset.v for details about setting this parameter
	parameter COUNT_106NS = 11 //Default: 100MHz clock
)(
    input clk,
	 input reset,
    input line_busy,
    input send_en,
    output rdy,
    output reg txenoob
    );

//States
parameter [1:0]
	s_idle = 2'b00,
	s_wait = 2'b01,
	s_assert = 2'b10;

reg [1:0] state, state_next;
reg [6:0] counter, counter_next;
reg [3:0] pulsecounter, pulsecounter_next;

always @ (*)
begin : comb_FSM
	//defaults
	state_next = state;
	counter_next = counter;
	pulsecounter_next = pulsecounter;
	
	case (state)
		s_idle:
			begin
				counter_next = 0;
				pulsecounter_next = 0;
				if (send_en == 1 && ~line_busy) begin
					state_next = s_wait;
				end
			end
		s_wait:
			begin
				counter_next = counter + 1;
				if (counter == COUNT_106NS) begin //11: 110ns at 100MHz
					counter_next = 0;
					//6 bursts
					pulsecounter_next = pulsecounter + 1;
					if (pulsecounter == 3'b101) //This is the sixth
						state_next = s_idle;
					else
						state_next = s_assert;
				end
			end
		s_assert:
			begin
				counter_next = counter + 1;
				if (counter == COUNT_106NS) begin
					state_next = s_wait;
					counter_next = 0;
				end
			end
		default: state_next = s_idle;
		
	endcase
end

always @ (posedge clk or posedge reset)
begin : seq_FSM
	if (reset) begin
		state <= s_idle;
		counter <= 0;
		pulsecounter <= 0;
	end
	else begin
		state <= state_next;
		counter <= counter_next;
		pulsecounter <= pulsecounter_next;
		txenoob <= (state==s_assert);
	end
end

//output
assign rdy = (state==s_idle && ~line_busy);
		

endmodule
